-----------------------------------------------------------------------------------------------
-- Client Lua Script for SimpleTimer
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- SimpleTimer Module Definition
-----------------------------------------------------------------------------------------------
local SimpleTimer = {}
local SavedTimer = {}
local DefaultSettings = {
	sDefaultTimerDuration = 30,
	sNotificationTimeout = 8,
	sCloseWndOnStart = false,
	sRestoreTimer = true
}
local Settings = DefaultSettings

-- get localization
local L = Apollo.GetPackage("Gemini:Locale-1.0").tPackage:GetLocale("SimpleTimer", true)

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SimpleTimer:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function SimpleTimer:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "SimpleTimer"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- SimpleTimer OnLoad
-----------------------------------------------------------------------------------------------
function SimpleTimer:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SimpleTimer.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
	self.xmlSettingsDoc = XmlDoc.CreateFromFile("SimpleTimerSettings.xml")
	self.xmlSettingsDoc:RegisterCallback("OnSettingsDocLoaded", self)
			
	-- timer(s)
	self.timer = ApolloTimer.Create(1, true, "OnTimer", self); self.isTimerRunning = false
	self.hideNotificationTimer = nil -- initialized in OnTimer() function for configuration purposes

end

-----------------------------------------------------------------------------------------------
-- SimpleTimer OnDocLoaded
-----------------------------------------------------------------------------------------------
function SimpleTimer:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "MainWindow", nil, self)
		self.wndNotification = Apollo.LoadForm(self.xmlDoc, "NotificationFrame", nil, self)

		if (self.wndMain == nil) then Apollo.AddAddonErrorText(self, "Could not load the main window for some reason."); return; end
				
	    self.wndMain:Show(false, true)
		self.wndNotification:Show(false, true)
		
		-- Register handlers for events, slash commands and timer, etc.
		Apollo.RegisterEventHandler("SimpleTimer_ShowHide", "OnInterfaceMenuShowHide", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		
		-- Restart a previously set timer if possible
		if SavedTimer.StartTime ~= nil and SavedTimer.Value ~= nil then
			self:OnRestart(SavedTimer.StartTime, SavedTimer.Value)
		else
			SavedTimer.StartTime = nil; SavedTimer.Value = nil
		end
		
		-- Texts
		--- Main timer window:
		self.wndMain:FindChild("TimerContent:TimerStatus"):SetText(L["notRunningStatus"])
		self.wndMain:FindChild("TimerContent:UserInput"):SetText(tostring(Settings.sDefaultTimerDuration))
		self.wndMain:FindChild("MenuContent:TimerButton"):SetText(L["mTimer"])
		self.wndMain:FindChild("MenuContent:AlarmButton"):SetText(L["mAlarm"])
		self.wndMain:FindChild("MenuContent:StopwatchButton"):SetText(L["mStopwatch"])
		self.wndMain:FindChild("PlaceholderNotice"):SetText(L["placeholderNotice"])
		--- Notification frame:
		self.wndNotification:FindChild("Content"):SetText(L["timerNotification"])
	end
end

function SimpleTimer:OnSettingsDocLoaded()
	if self.xmlSettingsDoc ~= nil and self.xmlSettingsDoc:IsLoaded() then
		self.wndConfig = Apollo.LoadForm(self.xmlSettingsDoc, "SimpleTimerSettingsForm", nil, self)
		self.wndConfigTimer = Apollo.LoadForm(self.xmlSettingsDoc, "TimerModule", self.wndConfig:FindChild("ContentTimer"), self)
		self.wndConfigAlarm = Apollo.LoadForm(self.xmlSettingsDoc, "AlarmModule", self.wndConfig:FindChild("ContentAlarm"), self)
		self.wndConfigStopw = Apollo.LoadForm(self.xmlSettingsDoc, "StopwModule", self.wndConfig:FindChild("ContentStopw"), self)
		
		-- Initialize config window on Timer tab.
		self.wndConfig:Show(false, true)
		self.wndConfig:FindChild("ContentTimer"):Show(true)
		self.wndConfig:FindChild("ContentAlarm"):Show(false)
		self.wndConfig:FindChild("ContentStopw"):Show(false)
		self.wndConfig:FindChild("ContentToggleContainer:TimerTab"):SetCheck(true)
		
		-- Register handlers for events, slash commands and timer, etc.
		Apollo.RegisterSlashCommand("simpletimer", "OnSlashCommand", self)
		Apollo.RegisterSlashCommand("st", "OnSlashCommand", self)

		-- Texts
		--- Tabs
		self.wndConfig:FindChild("ContentToggleContainer:TimerTab"):SetText(L["mTimer"])
		self.wndConfig:FindChild("ContentToggleContainer:AlarmTab"):SetText(L["mAlarm"])
		self.wndConfig:FindChild("ContentToggleContainer:StopwTab"):SetText(L["mStopwatch"])
		--- TimerTab
		self.wndConfigTimer:FindChild("DurationsFraming:Title"):SetText(L["sTDu_Title"])
		self.wndConfigTimer:FindChild("DurationsFraming:Description"):SetText(L["sTDu_Desc"])
		self.wndConfigTimer:FindChild("DurationsFraming:DefaultTimer:Label"):SetText(L["sTDu_DefaultTimerDuration"])
		self.wndConfigTimer:FindChild("DurationsFraming:DefaultNotification:Label"):SetText(L["sTDu_NotificationTimeout"])
		self.wndConfigTimer:FindChild("MiscFraming:Title"):SetText(L["sTMi_Title"])
		self.wndConfigTimer:FindChild("MiscFraming:AutoRestoreTimer"):SetText(L["sTMi_AutoRestoreTimer"])
		self.wndConfigTimer:FindChild("MiscFraming:CloseWndOnStart"):SetText(L["sTMi_CloseWndOnStart"])
		--- AlarmTab; StopwTab
		self.wndConfigAlarm:FindChild("PlaceholderFraming:Title"):SetText(L["sPlaceholder_Title"])
		self.wndConfigStopw:FindChild("PlaceholderFraming:Title"):SetText(L["sPlaceholder_Title"])
		self.wndConfigAlarm:FindChild("PlaceholderFraming:Description"):SetText(L["sPlaceholder_Desc"])
		self.wndConfigStopw:FindChild("PlaceholderFraming:Description"):SetText(L["sPlaceholder_Desc"])
	end
end

-----------------------------------------------------------------------------------------------
-- Saving and restoring
-----------------------------------------------------------------------------------------------
function SimpleTimer:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	return {
		sDefaultTimerDuration = Settings.sDefaultTimerDuration,
		sNotificationTimeout = Settings.sNotificationTimeout,
		sCloseWndOnStart = Settings.sCloseWndOnStart,
		sRestoreTimer = Settings.sRestoreTimer,
		bSavedTimer = { ["Value"] = SavedTimer.Value, ["StartTime"] = SavedTimer.StartTime },
	}
end

function SimpleTimer:OnRestore(eType, tData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end

	Settings.sDefaultTimerDuration = tData.sDefaultTimerDuration
	Settings.sNotificationTimeout = tData.sNotificationTimeout
	Settings.sCloseWndOnStart = tData.sCloseWndOnStart
	Settings.sRestoreTimer = tData.sRestoreTimer
	
	SavedTimer.StartTime = tData.bSavedTimer["StartTime"]
	SavedTimer.Value = tData.bSavedTimer["Value"]
end

-----------------------------------------------------------------------------------------------
-- SimpleTimer OnInterfaceMenulistHasLoaded
-----------------------------------------------------------------------------------------------
function SimpleTimer:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent(
		"InterfaceMenuList_NewAddOn",
		"Simple Timer", 
		{ "SimpleTimer_ShowHide", "", "" }
	)
end

-----------------------------------------------------------------------------------------------
-- SimpleTimer Functions
-----------------------------------------------------------------------------------------------
-- When the slashcommand is entered in chatbox
function SimpleTimer:OnSlashCommand(cmd, arg)
	if arg == "settings" or arg == "cfg" then
		self:OnConfigure()
	elseif tonumber(arg) ~= nil then
		self.secToWait = tonumber(arg) * 60
		SavedTimer.StartTime = os.time(); SavedTimer.Value = self.secToWait -- save current timer in case of ui reloading 
		self.timer:Start(); self.isTimerRunning = true; -- start the timer
	else
		self:OnInterfaceMenuShowHide()
	end
end

-- When we ask the timer to start (one way or another):
function SimpleTimer:OnTimer()
	-- If the timer is running, continue; else if it just reached 0, stop it and play a sound
	if (self.secToWait ~= nil and self.secToWait > 0) then -- if the timer is running
		self.statusMin = math.floor(self.secToWait / 60)
		self.statusSec = self.secToWait - (self.statusMin * 60)
		
		-- Add an extra 0 if printed number are below 10:
		if self.statusSec > 9 then 			self.wndMain:FindChild("TimerContent:TimerStatus"):SetText(self.statusMin .. ":" .. self.statusSec)
		else
			self.wndMain:FindChild("TimerContent:TimerStatus"):SetText(self.statusMin .. ":0" .. self.statusSec)
		end
		
		-- add an indicator on the InterfaceMenuList icon:
		if (self.secToWait % 60) == 0 then
			Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "Simple Timer", {true, nil, math.ceil(self.secToWait / 60)})
		end
		self.secToWait = self.secToWait-1
		
	elseif self.secToWait == 0 then -- if the timer just reached 0
		Sound.Play(198); self.wndNotification:Show(true)
		self.hideNotificationTimer = ApolloTimer.Create(Settings.sNotificationTimeout, false, "OnHideNotification", self); self.hideNotificationTimer:Start()		
		self.secToWait = nil
		self.wndMain:FindChild("TimerContent:TimerStatus"):SetText(L["notRunningStatus"])
		self.timer:Stop(); self.isTimerRunning = false
		-- Reset saved variables:
		SavedTimer.StartTime = nil
		SavedTimer.Value = nil
		-- Remove the indicator on the InterfaceMenuList icon:
		Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "Simple Timer", {false, nil, 0})
	end
end

-- On timer restart after ui reloading/relogging into the game:
function SimpleTimer:OnRestart(startTime, value)
	if startTime == nil or value == nil then return end
	if Settings.sRestoreTimer == false then return end
	
	ChatSystemLib.PostOnChannel(2, L["chatMessagePrefix"] .. L["timerResumeNotice"])
	self.secToWait = value - (os.time() - startTime)
	SavedTimer.StartTime = os.time(); SavedTimer.Value = self.secToWait -- save current timer in case of ui reloading 
	self.timer:Start(); self.isTimerRunning = true; -- then start the timer
end

-- On hide notification:
function SimpleTimer:OnHideNotification()
	self.wndNotification:Show(false)
end

-----------------------------------------------------------------------------------------------
-- MainWindow Functions
-----------------------------------------------------------------------------------------------
-- When the interface menu icon is clicked:
function SimpleTimer:OnInterfaceMenuShowHide()
	if self.wndMain:IsShown() then
		self.wndMain:Show(false, true)
	else
		self.wndMain:Show(true, true)
		self.wndMain:FindChild("MenuContent"):Show(true)
		self.wndMain:FindChild("MenuButton"):Show(false)
		self.wndMain:FindChild("TimerContent"):Show(false)
		self.wndMain:FindChild("PlaceholderNotice"):Show(false)
	end
end

function SimpleTimer:OnBackToMenu()
	self.wndMain:FindChild("MenuContent"):Show(true)
	self.wndMain:FindChild("MenuButton"):Show(false)
	self.wndMain:FindChild("PlaceholderNotice"):Show(false)
	self.wndMain:FindChild("TimerContent"):Show(false)
	self.wndMain:FindChild("AlarmContent"):Show(false)
	self.wndMain:FindChild("StopwatchContent"):Show(false)
end

function SimpleTimer:OnShowTimerWnd()
	self.wndMain:FindChild("MenuContent"):Show(false)
	self.wndMain:FindChild("MenuButton"):Show(true)
	self.wndMain:FindChild("TimerContent"):Show(true)
end

function SimpleTimer:OnShowAlarmWnd()
	self.wndMain:FindChild("MenuContent"):Show(false)
	self.wndMain:FindChild("MenuButton"):Show(true)
	self.wndMain:FindChild("PlaceholderNotice"):Show(true)
	self.wndMain:FindChild("AlarmContent"):Show(true)
end

function SimpleTimer:OnShowStopwatchWnd()
	self.wndMain:FindChild("MenuContent"):Show(false)
	self.wndMain:FindChild("MenuButton"):Show(true)
	self.wndMain:FindChild("PlaceholderNotice"):Show(true)
	self.wndMain:FindChild("StopwatchContent"):Show(true)
end

-- When the Start button is clicked:
function SimpleTimer:OnStart()		
	if self.secToWait ~= nil then self.secToWait = nil end -- (re)initialize the timer if it's not
	self.wndMain:FindChild("TimerContent:UserInput"):ClearFocus()
	self.userInput = self.wndMain:FindChild("TimerContent:UserInput"):GetText() -- get user input
	
	-- If user input is not a number, send an error; else start the timer normally:
	if tonumber(self.userInput) == nil then
		self.wndMain:FindChild("TimerContent:TimerStatus"):SetText(L["inputTypeError"])
	else
		self.secToWait = tonumber(self.userInput) * 60
		SavedTimer.StartTime = os.time(); SavedTimer.Value = self.secToWait -- save current timer in case of ui reloading 
		self.timer:Start(); self.isTimerRunning = true; -- start the timer
		
	end
	
	if Settings.sCloseWndOnStart == true then self.wndMain:Show(false); end
end

-----------------------------------------------------------------------------------------------
-- SimpleTimerSettingsForm Functions
-----------------------------------------------------------------------------------------------
-- When the configuration button is clicked or on slashcommand /s(imple)t(imer):
function SimpleTimer:OnConfigure()
	self.wndConfig:Show(true)
	self:OnConfigTimerTab() -- loading the Timer options tab by default
end

function SimpleTimer:OnConfigClose()
	self.wndConfig:Show(false)
end

function SimpleTimer:OnConfigTimerTab()
	-- Show current settings status: 
	self.wndConfigTimer:FindChild("DurationsFraming:DefaultTimer:Input"):SetText(tostring(Settings.sDefaultTimerDuration))
	self.wndConfigTimer:FindChild("DurationsFraming:DefaultNotification:Input"):SetText(tostring(Settings.sNotificationTimeout))
	self.wndConfigTimer:FindChild("MiscFraming:CloseWndOnStart"):SetCheck(Settings.sCloseWndOnStart)
	self.wndConfigTimer:FindChild("MiscFraming:AutoRestoreTimer"):SetCheck(Settings.sRestoreTimer)	
	-- Manage tabs visibility:
	self.wndConfig:FindChild("ContentTimer"):Show(true)
	self.wndConfig:FindChild("ContentAlarm"):Show(false)
	self.wndConfig:FindChild("ContentStopw"):Show(false)
	self.wndConfig:FindChild("ContentToggleContainer:TimerTab"):SetCheck(true)
	self.wndConfig:FindChild("ContentToggleContainer:AlarmTab"):SetCheck(false)
	self.wndConfig:FindChild("ContentToggleContainer:StopwTab"):SetCheck(false)
end

function SimpleTimer:OnConfigAlarmTab()
	-- Show current settings status: 
	--- Nothing ATM
	-- Manage tabs visibility:
	self.wndConfig:FindChild("ContentTimer"):Show(false)
	self.wndConfig:FindChild("ContentAlarm"):Show(true)
	self.wndConfig:FindChild("ContentStopw"):Show(false)
	self.wndConfig:FindChild("ContentToggleContainer:AlarmTab"):SetCheck(true)
	self.wndConfig:FindChild("ContentToggleContainer:TimerTab"):SetCheck(false)
	self.wndConfig:FindChild("ContentToggleContainer:StopwTab"):SetCheck(false)
	
end
function SimpleTimer:OnConfigStopwTab()
	-- Show current settings status: 
	--- Nothing ATM
	-- Manage tabs visibility:
	self.wndConfig:FindChild("ContentTimer"):Show(false)
	self.wndConfig:FindChild("ContentAlarm"):Show(false)
	self.wndConfig:FindChild("ContentStopw"):Show(true)
	self.wndConfig:FindChild("ContentToggleContainer:StopwTab"):SetCheck(true)
	self.wndConfig:FindChild("ContentToggleContainer:TimerTab"):SetCheck(false)
	self.wndConfig:FindChild("ContentToggleContainer:AlarmTab"):SetCheck(false)
end

-- TimerTab Settings:
function SimpleTimer:OnDefaultTimerDurationChanged()
	Settings.sDefaultTimerDuration = tonumber(self.wndConfigTimer:FindChild("DurationsFraming:DefaultTimer:Input"):GetText())
end

function SimpleTimer:OnNotificationTimeoutChanged()
	Settings.sNotificationTimeout = tonumber(self.wndConfigTimer:FindChild("DurationFraming:DefaultNotification:Input"):GetText())
end

function SimpleTimer:OnCloseWndOnStartToggle(wndHandler, wndControl, eMouseButton)
	Settings.sCloseWndOnStart = wndControl:IsChecked()
end

function SimpleTimer:OnAutoRestoreTimerToggle(wndHandler, wndControl, eMouseButton)
	Settings.sRestoreTimer = wndControl:IsChecked()
end

-- AlarmTab Settings:
-- StopwTab Settings:

-----------------------------------------------------------------------------------------------
-- SimpleTimer Instance
-----------------------------------------------------------------------------------------------
local SimpleTimerInst = SimpleTimer:new()
SimpleTimerInst:Init()
